package net.laparola.ui.android;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.actionbarsherlock.app.SherlockFragment;

import net.laparola.core.ComponenteInformazioni;
import net.laparola.core.Testi.TestoTipi;
import net.laparola.core.VersioneInformazioni;
import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.LaParolaUrl;
import net.laparola.ui.android.bibleview.BibleView;
import net.laparola.ui.android.bibleview.BibleView.OnBibleViewListener;
import android.os.Bundle;
import android.os.Handler;
import android.text.Editable;
import android.text.Html;
import android.text.style.URLSpan;
import android.text.Spanned;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout.LayoutParams;

public class LaParolaFragment extends SherlockFragment implements OnBibleViewListener {
	public class AnchorAndText {
		public String anchor;
		public String link;
		public String text;
		
		public AnchorAndText(String anchor, String link, String text) {
			this.anchor = anchor;
			this.link = link;
			this.text = text;
		}
	}
	
	public interface MyRunnable {
		public void run (LaParolaFragment self);
	}

	private BibleView bibleView;
	private LaParolaBrowser laParolaBrowser;
	private LaParolaActivity parent;
	private boolean mLoading;
	private int mSyncColor;
	
	public MyRunnable onCreateViewRunnable;
	private boolean mIgnoreNextUrlForSync;
	private Handler mHandler;
	private Runnable mSyncPanelsRunnable;
	public String onCreateGoToUrl;
	private boolean mCreated;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		parent = (LaParolaActivity)getActivity();
		
		mHandler = new Handler();
		mSyncPanelsRunnable = new Runnable() {
			@Override
			public void run() {
				parent.syncPanels(LaParolaFragment.this);
			}
		};
		
		bibleView = new BibleView(parent, null);
		bibleView.setParents((LaParolaActivity)getActivity(), this);
		
		//bibleView.setLayoutParams(new LayoutParams(LayoutParams.MATCH_PARENT, 0, 1));
		bibleView.setLayoutParams(new LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT));
		
		bibleView.setOnBibleViewListener(this);
		laParolaBrowser = bibleView.getBrowser();
		bibleView.setNightMode(parent.getNightMode());
				
		if (onCreateViewRunnable != null) {
			onCreateViewRunnable.run(this);
			onCreateViewRunnable = null;
		}
		
		mCreated = true;
		
		return bibleView;
	}

	public void onStartLoading(BibleView view) {
		mLoading = true;
		parent.onLoadingChanged();
	}

	public void onFinishedLoading(final BibleView view) {
		if (!parent.hasFinishedLoading()) {
			// potrebbe finire il caricamento prima di preparare
			// l'action bar
			view.postDelayed(new Runnable() {
				@Override
				public void run() {
					onFinishedLoading(view);
				}
			}, 100);
			return;
		}

		mLoading = false;
		
		parent.onLoadingChanged();
		parent.updateActionBar();
		
		if (!mIgnoreNextUrlForSync) {
			parent.syncPanels(this);
		} else {
			mIgnoreNextUrlForSync = false;
		}
	}

	@Override
	public void onAnchorChanged(BibleView bibleView) {
		mHandler.removeCallbacks(mSyncPanelsRunnable);
		mHandler.postDelayed(mSyncPanelsRunnable, 500);
	}

	public void setIgnoreNextUrlForSync(boolean value) {
		this.mIgnoreNextUrlForSync = value;
	}
	
	public void onTestiCambiati () {
		if (laParolaBrowser != null) {
			boolean ok = setVersione(laParolaBrowser.getVersione());
			if (!ok) {
				for (ComponenteInformazioni c : LaParolaBrowser.getTestiInstallati()) {
					if (c.getTipo().contains(TestoTipi.BIBBIA)) {
						setVersione(c.getComponente());
					}
					break;
				}
				//onVersionChanged(bibleView);
			}
		}
	}
	
	public void onVersionChanged(BibleView view) {
		parent.onVersionChanged();
	}

	@Override
	public void onZoomChanged(BibleView bibleView, int zoom) {
		parent.onZoomChanged(this, zoom);
	}
	
	public boolean isLoading() {
		return mLoading;
	}

	public int getSyncColor() {
		return mSyncColor;
	}

	public void setSyncColor(int mSyncColor) {
		this.mSyncColor = mSyncColor;
	}

	public List<AnchorAndText> getPlainTextWithAnchors() {
		Pattern rbody = Pattern.compile("<body.*?>(.*?)</body>", Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
		Pattern rcomment = Pattern.compile("<!--.*?-->", Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
		
		CharSequence testoCorrente = laParolaBrowser.getTestoCorrente();
		Matcher m = rbody.matcher(testoCorrente);
		if (m.find()) {
			testoCorrente = m.group(1);
		}
		
		m = rcomment.matcher(testoCorrente);
		testoCorrente = m.replaceAll("");
		
		Spanned html = Html.fromHtml(testoCorrente.toString());
		
		List<AnchorAndText> res = new ArrayList<AnchorAndText>();
		
		int i = 0;
		res.add(new AnchorAndText(null, null, null));
		for (URLSpan s : html.getSpans(0, html.length(), URLSpan.class)) {
			int start = html.getSpanStart(s);
			int end = html.getSpanEnd(s);

			res.get(res.size() - 1).text = html.subSequence(i, start).toString();
			res.add(new AnchorAndText(html.subSequence(start, end).toString(), s.getURL(), null));
			i = end;
		}
		if (res.size() > 0) {
			res.get(res.size() - 1).text = html.subSequence(i, html.length()).toString();
		}
		
		return res;
	}
	
	public boolean isCreated() {
		return mCreated;
	}
	
	// === Inizio proxy === 
	// TODO : ma ne vale la pena?
	// Il tutto è nato per essere sicuro che tutto fosse ok nel refactoring
	// della Activity in diversi Fragment, ma ora potrei rendere pubblici 
	// laParolaBrowser e bibleView.
	// D'altro canto, però, forse questo sistema è più "pulito".
	// Se Java avesse l'ereditarietà multipla, basterebbe far ereditare questa
	// classe da LaParolaBrowser.

	public boolean pageDown(boolean bottom) {
		return bibleView.pageDown(bottom);
	}

	public void goToNextUrl() {
		bibleView.goToNextUrl();
	}

	public boolean pageUp(boolean top) {
		return bibleView.pageUp(top);
	}

	public void goToPreviousUrl() {
		bibleView.goToPreviousUrl();
	}

	public void setNightMode(boolean value) {
		if (bibleView != null) {
			// altrimenti lo erediterà da parent
			bibleView.setNightMode(value);
		}
	}

	public int getTextZoom() {
		return bibleView.getTextZoom();
	}

	public void setTextZoom(int zoom, boolean showToast) {
		bibleView.setTextZoom(zoom, showToast);
	}
	
	public void setTextZoom(int zoom) {
		bibleView.setTextZoom(zoom);
	}
	
	public boolean precedenteEsiste() {
		return laParolaBrowser.precedenteEsiste();
	}

	public void vaiAPrecendente() {
		laParolaBrowser.vaiAPrecendente();
	}

	public String getVersione() {
		return laParolaBrowser.getVersione();
	}

	public LaParolaUrl getUrlCorrente() {
		return laParolaBrowser.getUrlCorrente();
	}

	public void vaiAHome() {
		laParolaBrowser.vaiAHome();
	}

	public void vaiAdUrl(String url) {
		laParolaBrowser.vaiAdUrl(url);
	}

	public void vaiAdUrl(LaParolaUrl url) {
		laParolaBrowser.vaiAdUrl(url);
	}

	public void vaiASuccessivo() {
		laParolaBrowser.vaiASuccessivo();
	}

	public void selectAndCopyText() {
		bibleView.selectAndCopyText();
	}

	public void aggiornaPagina() {
		if (laParolaBrowser == null)
			return;
					
		final LaParolaUrl urlCorrente = laParolaBrowser.getUrlCorrente();
		if (urlCorrente == null || urlCorrente.schema.equals("null")) {
			// workaround
			if (onCreateViewRunnable != null) {
				onCreateViewRunnable.run(this);
				onCreateViewRunnable = null;
			} else {
				laParolaBrowser.vaiAHome();
			}
		} else {
			laParolaBrowser.aggiornaPagina();
		}
	}

	public boolean setVersione(String nomeVersione) {
		return laParolaBrowser.setVersione(nomeVersione);
	}

	public boolean successivoEsiste() {
		return laParolaBrowser.successivoEsiste();
	}

	public boolean inHome() {
		return laParolaBrowser.inHome();
	}

	public void vaiARicerca(Editable text, Editable reference) {
		laParolaBrowser.vaiARicerca(text, reference);
	}

	public void vaiARicerca(Editable text) {
		laParolaBrowser.vaiARicerca(text);
	}

	public void vaiARiferimento(Editable ref) {
		laParolaBrowser.vaiARiferimento(ref);
	}

	public void vaiALibroCapitoloVersetto(int b, int c, int v) {
		laParolaBrowser.vaiALibroCapitoloVersetto(b, c, v);
	}

	public int getCapitoliInLibro(int b) {
		return laParolaBrowser.getCapitoliInLibro(b);
	}

	public Object getNomeLibro(int i) {
		return laParolaBrowser.getNomeLibro(i);
	}

	public int getVersettiInCapitolo(int book, int chapter) {
		return laParolaBrowser.getVersettiInCapitolo(book, chapter);
	}

	public VersioneInformazioni getInformazioniVersione() {
		return LaParolaBrowser.getInformazioniTesto(getVersione());
	}
}
