package net.laparola.ui.android.actionbar;

import com.actionbarsherlock.view.MenuItem;
import com.actionbarsherlock.view.MenuItem.OnActionExpandListener;
import net.laparola.R;
import net.laparola.ui.android.LaParolaActivity;
import net.laparola.ui.android.LaParolaFragment;
import android.content.Context;
import android.database.DataSetObserver;
import android.view.*;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.*;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.TextView.OnEditorActionListener;

public class ReferenceActionItemManager implements OnItemSelectedListener, OnActionExpandListener, OnClickListener, OnEditorActionListener {
    private LaParolaActivity parent;
    private MenuItem referenceActionItem;
    
    private LinearLayout bcvLayout;
    private Spinner bookSpinner;
    private Spinner chapterSpinner;
    private Spinner verseSpinner;
	private ImageButton referenceEditButton;
	private BookSpinnerAdapter bookSpinnerAdapter;
	private ChapterSpinnerAdapter chapterSpinnerAdapter;
	private VerseSpinnerAdapter verseSpinnerAdapter;
	private boolean ignoreBookSelection;
	private boolean ignoreChapterSelection;
	private boolean ignoreVerseSelection;
	
	private LinearLayout fullRefLayout;
	private EditText referenceEditText;
	private ImageButton referenceGoButton;
	
    
	public ReferenceActionItemManager(LaParolaActivity parent, MenuItem referenceActionItem) {
        this.parent = parent;
        this.referenceActionItem = referenceActionItem;
		LinearLayout referenceActionView = (LinearLayout)referenceActionItem.getActionView();
		
		bcvLayout = (LinearLayout)referenceActionView.findViewById(R.id.bcv_linear_layout);
		
		bookSpinner = (Spinner)referenceActionView.findViewById(R.id.book_spinner);
		bookSpinnerAdapter = new BookSpinnerAdapter();
		bookSpinner.setAdapter(bookSpinnerAdapter);
		bookSpinner.setOnItemSelectedListener(this);

		chapterSpinner = (Spinner)referenceActionView.findViewById(R.id.chapter_spinner);
		chapterSpinnerAdapter = new ChapterSpinnerAdapter();
		chapterSpinner.setAdapter(chapterSpinnerAdapter);
		chapterSpinner.setOnItemSelectedListener(this);
		
		verseSpinner = (Spinner)referenceActionView.findViewById(R.id.verse_spinner);
		verseSpinnerAdapter = new VerseSpinnerAdapter();
		verseSpinner.setAdapter(verseSpinnerAdapter);
		verseSpinner.setOnItemSelectedListener(this);
		
		bookSpinner.setSelection(0);
		chapterSpinner.setEnabled(false);
		verseSpinner.setEnabled(false);
		
		referenceEditButton = (ImageButton)referenceActionView.findViewById(R.id.reference_edit_btn);
		referenceEditButton.setOnClickListener(this);
		
		
		fullRefLayout = (LinearLayout)referenceActionView.findViewById(R.id.fullref_linear_layout);
		
		referenceEditText = (EditText)referenceActionView.findViewById(R.id.reference_edittext);
		referenceEditText.setOnEditorActionListener(this);
		
		referenceGoButton = (ImageButton)referenceActionView.findViewById(R.id.reference_go_btn);
		referenceGoButton.setOnClickListener(this);
		
		
		referenceActionItem.setOnActionExpandListener(this);
	}
	
	public void onClick(View view) {
		if (view == referenceEditButton) {
			bcvLayout.setVisibility(View.GONE);
			fullRefLayout.setVisibility(View.VISIBLE);
			
	        referenceEditText.post(new Runnable() {
	            public void run() {
	            	referenceEditText.requestFocusFromTouch();
	                InputMethodManager imm = (InputMethodManager)parent.getSystemService(Context.INPUT_METHOD_SERVICE);
	                imm.showSoftInput(referenceEditText, 0);
	            }
	        });
		} else if (view == referenceGoButton) {
			parent.getActiveFragment().vaiARiferimento(referenceEditText.getText());
		}
	}

    public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
        if ((event != null &&
             event.getAction() == KeyEvent.ACTION_DOWN && 
             event.getKeyCode() == KeyEvent.KEYCODE_ENTER) ||
            (actionId == EditorInfo.IME_ACTION_GO)) {
            
            parent.getActiveFragment().vaiARiferimento(referenceEditText.getText());
            return true;
        }

        return false;
    }	
    
    public void onItemSelected(AdapterView<?> view, View itemview, int position, long id) {
    	int b = (int)bookSpinner.getSelectedItemId();
    	int c = (int)chapterSpinner.getSelectedItemId();
    	int v = (int)verseSpinner.getSelectedItemId();
    	boolean load = false;        
        
        if (view == bookSpinner) {
        	if (!ignoreBookSelection) {
	        	chapterSpinnerAdapter.setBook(b);
	        	if (b == 0) {
	        		chapterSpinner.setEnabled(false);
	        		chapterSpinner.setSelection(0);
	        		verseSpinner.setEnabled(false);
	        		verseSpinner.setSelection(0);
	        	} else {
	        		chapterSpinner.setEnabled(true);
	        		chapterSpinner.setSelection(1);
	        		verseSpinner.setEnabled(true);
	        		verseSpinner.setSelection(1);
                    load = true;
	        	}
        	}
        	ignoreBookSelection = false;
        } else if (view == chapterSpinner) {
        	if (!ignoreChapterSelection) {
	    		verseSpinnerAdapter.setBookAndChapter(b, c);
	        	if (b == 0 || c == 0) {
	        		verseSpinner.setEnabled(false);
	        		verseSpinner.setSelection(0);
	        	} else {
	        		verseSpinner.setEnabled(true);
	        		verseSpinner.setSelection(1);
                    load = true;
	        	}
        	}
        	ignoreChapterSelection = false;
        } else if (view == verseSpinner) {
        	if (!ignoreVerseSelection) {
                load = true;
	    	}
        	ignoreVerseSelection = false;
        }
        
        if (load && b != 0 && c != 0 && v != 0) {
            parent.getActiveFragment().vaiALibroCapitoloVersetto(b, c, v);
        }
    }

    public void onNothingSelected(AdapterView<?> arg0) {}

    public boolean onMenuItemActionExpand(MenuItem item) {
        return parent.collapseActionViewsExcept(item);
    }
    
    public boolean onMenuItemActionCollapse(MenuItem item) {
		if (bcvLayout.getVisibility() == View.GONE) {
			bcvLayout.setVisibility(View.VISIBLE);
			fullRefLayout.setVisibility(View.GONE);
	        
			InputMethodManager imm = (InputMethodManager)parent.getSystemService(Context.INPUT_METHOD_SERVICE);
	        imm.hideSoftInputFromWindow(referenceEditText.getWindowToken(), 0);
	        referenceEditText.post(new Runnable() {
	            public void run() {
	            	referenceEditText.clearFocus();
	            }
	        });
	        
	        // preferisco chiudere del tutto il reference
	        //return false;
		}
        return true;
    }
    
    public void onVersionChanged() {
		updateBooks();
	}

	public void collapse (MenuItem exclude) {
        if (exclude != referenceActionItem) {
            referenceActionItem.collapseActionView();
        }
    }
    
    public void select (int b, int c, int v) {
    	chapterSpinnerAdapter.setBook(b);
		verseSpinnerAdapter.setBookAndChapter(b, c); 
		
        if (bookSpinner.getSelectedItemId() != b) {
        	ignoreBookSelection = true;
        	boolean ok = false;
	        for (int pos = 0; pos < bookSpinnerAdapter.getCount(); pos++) {
	        	if (bookSpinnerAdapter.getItemId(pos) == b) { 
	        		bookSpinner.setSelection(pos);
	        		ok = true;
	        		break;
	        	}
	        }
	        if (ok == false) {
	        	b = (int)bookSpinner.getSelectedItemId();
	        	c = (b != 0) ? 1 : 0;
	        	v = c;
	        }
        }
        
        if (b != 0 && c != 0 && v != 0) {
            chapterSpinner.setEnabled(true);
            verseSpinner.setEnabled(true);
    	}
        
        if (chapterSpinner.getSelectedItemId() != c) {
        	ignoreChapterSelection = true;
	        chapterSpinner.setSelection(c);
        } 

        if (verseSpinner.getSelectedItemId() != v) {
        	ignoreVerseSelection = true;
	        verseSpinner.setSelection(v);
        }
    }
    
    public void select (CharSequence reference) {
    	referenceEditText.setText(reference);
    }

    private class BookSpinnerAdapter extends BCSSpinnerAdapter {
    	private int[] book_ids;
    	private int book_count;
    	
    	public BookSpinnerAdapter () {
    		super();
    		
    		book_ids = new int[74];
    		updateBooks();
    	}
    	
    	public void updateBooks () {
    		book_count = 1;   // compreso lo 0 (nessun libro)

			LaParolaFragment fragment = getFragment();
    		if (fragment != null && fragment.isVisible() && fragment.getVersione().length() != 0) {
				for (int b = 1; b <= 73; b++) {
	    			if (getFragment().getCapitoliInLibro(b) != 0) {
	    				book_ids[book_count++] = b;
					}
	    		}
    		} else {
				// non ci sono versioni installate, metto tutti i libri
        		for (int b = 1; b <= 73; b++) {
      				book_ids[book_count++] = b;
        		}
    		}
    	}
    	    	
		public int getCount() {
			return book_count;
		}

		public Object getItem(int position) {
			LaParolaFragment fragment = getFragment();
			if (position == 0 || fragment == null) return "";
			return fragment.getNomeLibro(book_ids[position]);
		}

		public long getItemId(int position) {
			if (position == 0) return 0;
			return book_ids[position];
		}
		
		@Override
		public View getDropDownView(int position, View convertView, ViewGroup genitore) {
			TextView text = (TextView)super.getDropDownView(position, convertView, genitore);
	        boolean empty = text.getText().length() == 0;
			text.setEnabled(!empty);
			text.setClickable(empty);   // controintuitivo... perchè? forse ha a che fare con 
										// il fatto che si clicca sul contenitore
			if (empty) text.setText(R.string.book);
			return text;
		}
		
		@Override
		public View getView(int position, View convertView, ViewGroup genitore) {
			TextView text = (TextView)super.getView(position, convertView, genitore);
	        boolean empty = text.getText().length() == 0;
			if (empty) text.setText(R.string.book);
			return text;
		}
    }

    private class ChapterSpinnerAdapter extends BCSSpinnerAdapter {
    	private int book;
    	
    	public ChapterSpinnerAdapter () {
    		super();
    		book = 0;
    	}
    	
    	public void setBook (int b) {
    		book = b;
    	}
    	
		public int getCount() {
			LaParolaFragment fragment = getFragment();
			if (book == 0 || fragment == null) return 1;
			return Math.max(1, fragment.getCapitoliInLibro(book) + 1);
		}

		public Object getItem(int position) {
			if (position == 0) return ""; 
			return String.valueOf(position);
		}

		public long getItemId(int position) {
			return position;
		}
    }
    
    private class VerseSpinnerAdapter extends BCSSpinnerAdapter {
    	private int book;
    	private int chapter;
    	
    	public VerseSpinnerAdapter () {
    		super();
    		book = 0;
    		chapter = 0;
    	}
  
    	public void setBookAndChapter (int b, int c) {
    		book = b;
    		chapter = c;
    	}
    	
		public int getCount() {
			LaParolaFragment fragment = getFragment();
			if (book == 0 || chapter == 0 || fragment == null) return 1;
			return Math.max(1,  fragment.getVersettiInCapitolo(book, chapter) + 1);
		}

		public Object getItem(int position) {
			if (position == 0) return ""; 
			return String.valueOf(position);
		}

		public long getItemId(int position) {
			return position;
		}
    }
    
    private abstract class BCSSpinnerAdapter implements SpinnerAdapter {
    	private LayoutInflater mInflater;
    	
    	public LaParolaFragment getFragment () {
    		return parent.getActiveFragment();
    	}
    	
		public int getItemViewType(int position) {
			return 0;
		}

		public int getViewTypeCount() {
			return 1;
		}

		public boolean hasStableIds() {
			return true;
		}

		public boolean isEmpty() {
			return false;
		}

		public void registerDataSetObserver(DataSetObserver observer) {}

		public void unregisterDataSetObserver(DataSetObserver observer) {}
		
	    protected View createViewFromResource(int position, View convertView, ViewGroup parent, int resource) {
	        View view;
	        TextView text;

	        if (convertView == null) {
	        	if (mInflater == null) {
	        		mInflater = (LayoutInflater)parent.getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	        	}
	            view = mInflater.inflate(resource, parent, false);
	        } else {
	            view = convertView;
	        }

	        try {
	            int mFieldId = 0;
	            
	        	if (mFieldId == 0) {
	                //  If no custom field is assigned, assume the whole resource is a TextView
	                text = (TextView) view;
	            } else {
	                //  Otherwise, find the TextView field within the layout
	                text = (TextView) view.findViewById(mFieldId);
	            }
	        } catch (ClassCastException e) {
	            throw new IllegalStateException("ArrayAdapter requires the resource ID to be a TextView", e);
	        }

	        Object item = getItem(position);
	        if (item instanceof CharSequence) {
	            text.setText((CharSequence)item);
	        } else {
	            text.setText(item.toString());
	        }
	        
	        return view;
	    }		
	    
		public View getView(int position, View convertView, ViewGroup parent) {
			return createViewFromResource(position, convertView, parent, android.R.layout.simple_spinner_item);
		}

		public View getDropDownView(int position, View convertView,	ViewGroup parent) {
			return createViewFromResource(position, convertView, parent, android.R.layout.simple_spinner_dropdown_item);
		}
    }

	public void expand() {
		referenceActionItem.expandActionView();
	}

	public void updateBooks() {
		bookSpinnerAdapter.updateBooks();
	}
}
