package net.laparola.ui.android.actionbar.view.ignspinner;

import net.laparola.core.Testi.TestoTipi;
import net.laparola.ui.android.actionbar.view.IgnPopupWindow;
import net.laparola.ui.android.library.LibraryActivity;
import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListAdapter;
import android.widget.ListView;

import com.actionbarsherlock.R;

class IgnDropDownListView extends LinearLayout implements OnClickListener {
	public interface OnTypeChangedListener {
		public void OnTypeChanged (TestoTipi tipo);
	}
	
    /*
     * WARNING: This is a workaround for a touch mode issue.
     *
     * Touch mode is propagated lazily to windows. This causes problems in
     * the following scenario:
     * - Type something in the AutoCompleteTextView and get some results
     * - Move down with the d-pad to select an item in the list
     * - Move up with the d-pad until the selection disappears
     * - Type more text in the AutoCompleteTextView *using the soft keyboard*
     *   and get new results; you are now in touch mode
     * - The selection comes back on the first item in the list, even though
     *   the list is supposed to be in touch mode
     *
     * Using the soft keyboard triggers the touch mode change but that change
     * is propagated to our window only after the first list layout, therefore
     * after the list attempts to resurrect the selection.
     *
     * The trick to work around this issue is to pretend the list is in touch
     * mode when we know that the selection should not appear, that is when
     * we know the user moved the selection away from the list.
     *
     * This boolean is set to true whenever we explicitly hide the list's
     * selection and reset to false whenever we know the user moved the
     * selection back to the list.
     *
     * When this boolean is true, isInTouchMode() returns true, otherwise it
     * returns super.isInTouchMode().
     */
    boolean mListSelectionHidden;

    private boolean mHijackFocus;
    
    private ListView mListView;
    private LinearLayout mButtons;
	private Button mBibleButton;
	private Button mCommentarioButton;
	private View mSeparator;
	private Button mLibraryButton;

	private OnTypeChangedListener mOnTypeChangedListener;
    private AdapterView.OnItemSelectedListener mOnItemSelectedListener;
	
    public IgnDropDownListView(Context context) {
    	this(context, false);
    }
    
    public IgnDropDownListView(Context context, boolean hijackFocus) {
        super(context);

        inflate(context, R.layout.version_chooser, this);
        mButtons = (LinearLayout)findViewById(R.id.buttons);
        mBibleButton = (Button)mButtons.findViewById(R.id.button_bible);
        mCommentarioButton = (Button)mButtons.findViewById(R.id.button_commentario);
        mBibleButton.setOnClickListener(this);
        mCommentarioButton.setOnClickListener(this);
        mListView = (ListView)findViewById(R.id.list_view);
        mSeparator = findViewById(R.id.separator);
        mLibraryButton = (Button)findViewById(R.id.library_button);
        mLibraryButton.setOnClickListener(this);
        
        mHijackFocus = hijackFocus;
        // TODO: Add an API to control this
        mListView.setCacheColorHint(0); // Transparent, since the background drawable could be anything.
    }

	@Override
	public void onClick(View v) {
		if (v == mLibraryButton) {
			Context context = getContext();
			Intent intent = new Intent(context, LibraryActivity.class);
			context.startActivity(intent);			
			mOnItemSelectedListener.onNothingSelected(null);
			return;
		}
		
		// workaround per un bug
		((Button)mBibleButton).setGravity(Gravity.NO_GRAVITY);
		((Button)mBibleButton).setGravity(Gravity.CENTER);
		
		if (v.isSelected()) {
			return;
		}
		
		TestoTipi r = TestoTipi.BIBBIA;
		if (v == mBibleButton) {
			r = TestoTipi.BIBBIA;
		} else if (v == mCommentarioButton) {
			r = TestoTipi.COMMENTARIO;
		}
		
		setSelectedType(r);
		if (mOnTypeChangedListener != null) {
			mOnTypeChangedListener.OnTypeChanged(r);
		}
	}
	
	@Override
    public boolean isInTouchMode() {
        // WARNING: Please read the comment where mListSelectionHidden is declared
        return (mHijackFocus && mListSelectionHidden) || super.isInTouchMode();
    }

    @Override
    public boolean hasWindowFocus() {
        return mHijackFocus || super.hasWindowFocus();
    }

    @Override
    public boolean isFocused() {
        return mHijackFocus || super.isFocused();
    }

    @Override
    public boolean hasFocus() {
        return mHijackFocus || super.hasFocus();
    }

	public void setOnScrollListener(IgnPopupWindow.PopupScrollListener listener) {
		mListView.setOnScrollListener(listener);
	}

	public void setOnItemSelectedListener(OnItemSelectedListener listener) {
		mListView.setOnItemSelectedListener(listener);
        mOnItemSelectedListener = listener;
	}
	
	public void setAdapter(ListAdapter mAdapter) {
		mListView.setAdapter(mAdapter);
	}

	public ListView getListView() {
		return mListView;
	}

	public void setOnItemClickListener(OnItemClickListener listener) {
		mListView.setOnItemClickListener(listener);
	}

	public void setSelector(Drawable mDropDownListHighlight) {
		mListView.setSelector(mDropDownListHighlight);
	}

	public int getCount() {
		return mListView.getCount();
	}

	public void setSelection(int position) {
		mListView.setSelection(position);
	}
	
    public int measureHeightOfChildren(int widthMeasureSpec, int startPosition, int endPosition,
            final int maxHeight, int disallowPartialChildPosition) {

    	int buttonsHeight = 0;
    	if (mButtons.getVisibility() == View.VISIBLE) {
        	mBibleButton.measure(widthMeasureSpec, LayoutParams.WRAP_CONTENT);
        	mSeparator.measure(widthMeasureSpec, 0);
        	mLibraryButton.measure(widthMeasureSpec, 0);
    		buttonsHeight = mBibleButton.getMeasuredHeight();
    		buttonsHeight += mSeparator.getMeasuredHeight();
    		buttonsHeight += mLibraryButton.getMeasuredHeight();
    	}
    	
        final ListAdapter adapter = mListView.getAdapter();
        if (adapter == null) {
            return buttonsHeight + mListView.getListPaddingTop() + mListView.getListPaddingBottom();
        }

        // Include the padding of the list
        int returnedHeight = mListView.getListPaddingTop() + mListView.getListPaddingBottom();
        final int dividerHeight = ((mListView.getDividerHeight() > 0) && mListView.getDivider() != null) ? mListView.getDividerHeight() : 0;
        // The previous height value that was less than maxHeight and contained
        // no partial children
        int prevHeightWithoutPartialChild = 0;
        int i;
        View child;

        // mItemCount - 1 since endPosition parameter is inclusive
        endPosition = (endPosition == -1/*NO_POSITION*/) ? adapter.getCount() - 1 : endPosition;

        for (i = startPosition; i <= endPosition; ++i) {
            child = adapter.getView(i, null, mListView);
            if (mListView.getCacheColorHint() != 0) {
                child.setDrawingCacheBackgroundColor(mListView.getCacheColorHint());
            }

            measureScrapChild(child, i, widthMeasureSpec);

            if (i > 0) {
                // Count the divider for all but one child
                returnedHeight += dividerHeight;
            }

            returnedHeight += child.getMeasuredHeight();

            if (returnedHeight >= maxHeight) {
                // We went over, figure out which height to return.  If returnedHeight > maxHeight,
                // then the i'th position did not fit completely.
                return (disallowPartialChildPosition >= 0) // Disallowing is enabled (> -1)
                            && (i > disallowPartialChildPosition) // We've past the min pos
                            && (prevHeightWithoutPartialChild > 0) // We have a prev height
                            && (returnedHeight != maxHeight) // i'th child did not fit completely
                        ? prevHeightWithoutPartialChild
                        : maxHeight;
            }

            if ((disallowPartialChildPosition >= 0) && (i >= disallowPartialChildPosition)) {
                prevHeightWithoutPartialChild = returnedHeight;
            }
        }

        // At this point, we went through the range of children, and they each
        // completely fit, so return the returnedHeight
        return buttonsHeight + returnedHeight;
    }
    
    private void measureScrapChild(View child, int position, int widthMeasureSpec) {
        ListView.LayoutParams p = (ListView.LayoutParams) child.getLayoutParams();
        if (p == null) {
            p = new ListView.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT, 0);
            child.setLayoutParams(p);
        }
        //XXX p.viewType = mAdapter.getItemViewType(position);
        //XXX p.forceAdd = true;

        int childWidthSpec = ViewGroup.getChildMeasureSpec(widthMeasureSpec,
        		mListView.getPaddingLeft() + mListView.getPaddingRight(), p.width);
        int lpHeight = p.height;
        int childHeightSpec;
        if (lpHeight > 0) {
            childHeightSpec = MeasureSpec.makeMeasureSpec(lpHeight, MeasureSpec.EXACTLY);
        } else {
            childHeightSpec = MeasureSpec.makeMeasureSpec(0, MeasureSpec.UNSPECIFIED);
        }
        child.measure(childWidthSpec, childHeightSpec);
    }

	public OnTypeChangedListener getOnTypeChangedListener() {
		return mOnTypeChangedListener;
	}

	public void setOnTypeChangedListener(OnTypeChangedListener listener) {
		mOnTypeChangedListener = listener;
	}

	public void setSelectedType(final TestoTipi tipo) {
		mButtons.post(new Runnable() {
			@Override
			public void run() {
				switch (tipo) {
				case BIBBIA:
				case DIZIONARIO:
				case LIBRO:
				case NESSUNO:
					mBibleButton.setSelected(true);
					mCommentarioButton.setSelected(false);
					break;
				case COMMENTARIO:
					mBibleButton.setSelected(false);
					mCommentarioButton.setSelected(true);
					break;
				}
			}
		});
	}
}