package net.laparola.ui.android.bibleview;

// ricerca nel brano:
// findAll("parola");
// Method m = WebView.class.getMethod("setFindIsUp", Boolean.TYPE);
// m.invoke(webView, true);
// boolean ok = findNext();

import java.lang.reflect.Method;
import java.net.URLDecoder;

import net.laparola.R;
import net.laparola.ui.LaParolaBrowser;
import net.laparola.ui.LaParolaUrl;
import net.laparola.ui.LaParolaBrowser.LaParolaBrowserClient;
import net.laparola.ui.android.LaParolaActivity;
import net.laparola.ui.android.LaParolaFragment;
import net.laparola.ui.android.LaParolaJavascriptInterface;
import net.laparola.ui.android.library.LibraryActivity;

import android.annotation.SuppressLint;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Parcelable;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ContextMenu;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.KeyEvent.DispatcherState;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

@SuppressLint("SetJavaScriptEnabled")
public class BibleView extends WebView implements LaParolaBrowserClient {
	class BibleWebViewClient extends WebViewClient {
		@Override
		public boolean shouldOverrideUrlLoading(WebView view, String url) {
			if (isFrozen())
				return true;

			try {
				url = URLDecoder.decode(url, "UTF-8");
			} catch (Exception e) {
				//
			}
			LaParolaUrl purl = LaParolaBrowser.nuovoUrl(url, mLaParolaBrowser.getVersione());
			if (purl.gestito) {
				if (purl.richiedeNuovaFinestra(mLaParolaBrowser.getUrlCorrente()) && mFragment != null) {
					mParentActivity.openInPopupWindow(purl);
				} else {
					mLaParolaBrowser.vaiAdUrl(purl);
				}
				return true;
			}
			return false;
		}

		@Override
		public void onPageFinished(WebView view, String url) {
			final BibleView bibleView = (BibleView)view;
			if (mGoToAnchor != null) {
				bibleView.vaiAdAncoraggio(mGoToAnchor);
				mGoToAnchor = null;
			} else {
				finishedLoading();
			}
			bibleView.executeJavascript("setupOnScroll()");
		}
	}

	public interface OnBibleViewListener {
		public void onStartLoading(BibleView view);

		public void onFinishedLoading(BibleView view);

		public void onVersionChanged(BibleView view);

		public void onZoomChanged(BibleView bibleView, int mTextZoom);

		public void onAnchorChanged(BibleView bibleView);
	}

	private LaParolaBrowser mLaParolaBrowser;
	private OnBibleViewListener mOnBibleViewListener = null;
	private String mGoToAnchor;
	private Context mContext;
	private boolean mIsFrozen;
	private WebSettings mSettings;
	private int mTextZoom = 100;
	private BibleViewTouchHandler mTouchHandler;
	private Toast mZoomToast;
	private boolean mNightMode;
	private boolean mLoading;
	private boolean mIgnoreNextScroll;
	private LaParolaActivity mParentActivity;
	private LaParolaFragment mFragment;
	protected boolean mSkipNextContextMenu;

	public BibleView(Context context, AttributeSet attrs) {
		super(context, attrs);
		mContext = context;
		
		if (mContext instanceof Activity)
			((Activity)mContext).registerForContextMenu(this);

		setWebViewClient(new BibleWebViewClient());
		mSettings = getSettings();
		mSettings.setJavaScriptEnabled(true);
		mSettings.setBuiltInZoomControls(false);

		mLaParolaBrowser = new LaParolaBrowser();
		mLaParolaBrowser.setLaParolaBrowserClient(this);

		mTouchHandler = new BibleViewTouchHandler(this);

		addJavascriptInterface(new LaParolaJavascriptInterface(this), "LaParola");

		mZoomToast = Toast.makeText(mContext, "", Toast.LENGTH_SHORT);
	}

	@Override
	protected void onSizeChanged(int w, int h, int ow, int oh) {
		super.onSizeChanged(w, h, ow, oh);
		executeJavascript("setupOnScroll()");
	}
	
	@Override
	public boolean performLongClick() {
		if (mTouchHandler.isZooming()) {
			return false;
		}
		return super.performLongClick();
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent ev) {
		if ((ev.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_DOWN && ev.getPointerCount() == 1) {
			if (mFragment != null) {
				mParentActivity.onFragmentTouch(mFragment);
			}
		}
		
		if (mTouchHandler.onTouchEvent(ev))
			return true;

		return super.onTouchEvent(ev);
	}
	
	public void setParents(LaParolaActivity parentActivity, LaParolaFragment fragment) {
		mParentActivity = parentActivity;
		mFragment = fragment;		
	}
	
	@Override
	protected void onRestoreInstanceState(Parcelable state) {
		super.onRestoreInstanceState(state);
		mLaParolaBrowser.setLaParolaBrowserClient(this);
	}
	
	@Override
	protected Parcelable onSaveInstanceState() {
		mLaParolaBrowser.setLaParolaBrowserClient(null);
		return super.onSaveInstanceState();
	}

	public void setOnBibleViewListener(OnBibleViewListener onBibleViewListener) {
		this.mOnBibleViewListener = onBibleViewListener;
	}

	@TargetApi(8)
	@SuppressWarnings("deprecation")
	public void selectAndCopyText() {
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
        	mSkipNextContextMenu = true;
        	BibleView.super.performLongClick();
		} else if (Build.VERSION.SDK_INT >= 8) {
			emulateShiftHeld();
		} else {
			try {
				Method m = WebView.class.getDeclaredMethod("emulateShiftHeld");
				m.setAccessible(true);
				m.invoke(this);
			} catch (Exception e) {
				// fallback
				KeyEvent shiftPressEvent = new KeyEvent(0, 0, KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_SHIFT_LEFT, 0, 0);
				shiftPressEvent.dispatch(this, new DispatcherState(), this);
			}
		}
	}

	public LaParolaBrowser getBrowser() {
		return mLaParolaBrowser;
	}

	@Override
	public void visualizzaTesto(final CharSequence testo, final LaParolaUrl url) {
		post(new Runnable() {
			@Override
			public void run() {
				if (testo != null) {
					loadDataWithBaseURL(url.getUrl(), testo.toString(), "text/html", "UTF-8", url.getUrl());
				} else {
					loadDataWithBaseURL(url.getUrl(), "", "text/html", "UTF-8", url.getUrl());
				}
				mGoToAnchor = url.ancoraggio;
				
				if (mFragment == null) {
					setBackgroundColor(0);
					android.util.Log.d("laparola", "*");
				}
			}
		});
	}

	@Override
	public void vaiAdAncoraggio(final String ancoraggio) {
		post(new Runnable() {
			@Override
			public void run() {
				mIgnoreNextScroll = true;
				executeJavascript("location.hash = '#" + ancoraggio + "'");
				finishedLoading();
			}
		});
	}

	@Override
	public void onRichiestaIniziata(final LaParolaUrl url) {
		post(new Runnable() {
			@Override
			public void run() {
				Log.d("laparola", "Iniziato caricamento di " + url.getUrl());
				mLoading = true;
				if (mOnBibleViewListener != null) {
					mOnBibleViewListener.onStartLoading(BibleView.this);
				}
			}
		});
	}

	@Override
	public void onVersioneCambiata() {
		post(new Runnable() {
			@Override
			public void run() {
				if (mOnBibleViewListener != null) {
					mOnBibleViewListener.onVersionChanged(BibleView.this);
				}
			}
		});
	}

	@Override
	public void apriGestioneVersioni() {
		Intent intent = new Intent(mContext, LibraryActivity.class);
		mContext.startActivity(intent);
	}

	@Override
	public void visualizzaSito() {
		apriLink(mContext.getString(R.string.laparola_url));
	}

	@Override
	public void apriLink(String link) {
		LaParolaActivity.apriLink(mContext, link);
	}

	public void setFrozen(boolean b) {
		mIsFrozen = b;
	}

	public boolean isFrozen() {
		return mIsFrozen;
	}

	public void superOnTouchEvent(MotionEvent ev) {
		super.onTouchEvent(ev);
	}

	public void setTextZoom(int zoom) {
		setTextZoom(zoom, true);
	}

	public void setTextZoom(int zoom, boolean showToast) {
		if (zoom < 10)
			zoom = 10;
		if (zoom > 500)
			zoom = 500;

		if (zoom != mTextZoom) {
			mTextZoom = zoom;
			executeJavascript("document.body.style.fontSize = '" + String.valueOf(mTextZoom) + "%';");
	
			if (showToast) {
				mZoomToast.setText(mContext.getString(R.string.zoom_percent, mTextZoom));
				mZoomToast.show();
			}
			
			mOnBibleViewListener.onZoomChanged(this, mTextZoom);
		}
	}

	public void executeJavascript(String command) {
		loadUrl("javascript:(function() {" + command + "})()");
	}

	public int getTextZoom() {
		return mTextZoom;
	}

	public void cancelTouch(MotionEvent ev) {
		// serve ad evitare il longpress quando si fa lo zoom multitouch
		// cancelLongPress non basta perché comunque il rettangolo di 
		// selezione viene mantenuto
		
		try {
			// Si, lo so, non si accede ai metodi privati... 
			// Ma non c'è altro modo!
			
			// Android 2.1-4.1.1
			Object w;
			if (Build.VERSION.SDK_INT <= 15) {
				w = this;
			} else {
				w = WebView.class.getDeclaredMethod("getWebViewProvider").invoke(this);
			}
			
			Method m = w.getClass().getDeclaredMethod("cancelTouch");
			m.setAccessible(true);
			m.invoke(w);
			
		} catch (Exception e) {
			// Non funziona del tutto (non toglie il rettangolo), ma dovrebbe essere eseguito solo per
			// Android < 2.1 (non multitouch, e comunque non serve) ed eventualmente > 4.1.1...
			
			this.cancelLongPress();
			
			// togliendo il commento manda un annulla anche allo scalegesturedetector
			// forse si potrebbe risolvere mettendo speciali coordinate da 
			// ignorare in scalegesturedetector... 
			// ci vediamo quando uscirà android 4.2 (ed il nuovo webviewprovider di 
			// chrome, che magari non ha bisogno del cancel)
			
			// MotionEvent nev = MotionEvent.obtain(ev);
			// nev.setAction(MotionEvent.ACTION_CANCEL);
			// super.dispatchTouchEvent(nev);
		}
	}

	@Override
	public CharSequence getAggiuntaHeader(LaParolaUrl url) {
		StringBuilder sb = new StringBuilder();

        sb.append("<script type='text/javascript' src='file:///android_asset/utils.js'></script>");
		sb.append("<style type='text/css'><!--\n");
		appendCSS(sb);
		sb.append("--></style>");

		return sb;
	}

	private StringBuilder appendCSS(StringBuilder sb) {
		if (sb == null)
			sb = new StringBuilder();
		
		String background, bodycolor, acolor, versettocolor;
		
		if (!mNightMode) {
			 if (mFragment != null) {
 				 background = "white";
 			 } else {
				 background = "transparent";
 			 }
			 bodycolor = "black";
			 acolor = "blue";
			 versettocolor = "black";
		} else {
			 background = "black";
			 bodycolor = "white";
			 acolor = "yellow";
			 versettocolor = "white";
		}
		
		sb.append("  body {\n");
		sb.append("    font-size:  ").append(mTextZoom).append("%;\n");
		sb.append("    background: ").append(background).append(";\n");
		sb.append("    color:      ").append(bodycolor).append(";\n");
		sb.append("  }\n");
		sb.append("  a {\n");
		sb.append("    color:      ").append(acolor).append(";\n");
		sb.append("  }\n");
		sb.append("  a.versetto {\n");
		sb.append("    color:      ").append(versettocolor).append(";\n");
		sb.append("  }\n");
		
		return sb;
	}

	public void goToNextUrl() {
		if (mLaParolaBrowser.getUrlCorrente() != null) {
			String url = mLaParolaBrowser.getUrlCorrente().getUrlSuccessivo();
			if (url != null)
				mLaParolaBrowser.vaiAdUrl(url);
		}
	}

	public void goToPreviousUrl() {
		if (mLaParolaBrowser.getUrlCorrente() != null) {
			String url = mLaParolaBrowser.getUrlCorrente().getUrlPrecedente();
			if (url != null)
				mLaParolaBrowser.vaiAdUrl(url);
		}
	}

	public void setNightMode(boolean value) {
		mNightMode = value;

		StringBuilder css = appendCSS(null);
		applyCSS(css);
	}

	public void applyCSS(CharSequence css) {
		StringBuilder sb = new StringBuilder();
		sb.append("var styleElement = document.createElement(\"style\");");
		sb.append("styleElement.type = \"text/css\";");
		sb.append("styleElement.appendChild(document.createTextNode('");
		sb.append(css);
		sb.append("'));");
		sb.append("document.getElementsByTagName(\"head\")[0].appendChild(styleElement);");
		
		int i = sb.indexOf("\n");
		while (i != -1) {
			sb.replace(i, i + 1, " ");
			i = sb.indexOf("\n");
		}
		
		executeJavascript(sb.toString());
	}

	public boolean getNightMode() {
		return mNightMode;
	}

	public void onScrolledToAnchor(String s) {
		if (mIgnoreNextScroll) {
			mIgnoreNextScroll = false;
		} else if (!mLoading) {
			getBrowser().getUrlCorrente().ancoraggio = s;
			mOnBibleViewListener.onAnchorChanged(this);
		}
	}

	private void finishedLoading() {
		if (mLoading) {
			mLoading = false;
            
            StringBuilder css = appendCSS(null);
            applyCSS(css);
        
			if (mFragment == null)
				setVisibility(View.VISIBLE);   // per evitare glitch nella finestra popup
			
			if (mOnBibleViewListener != null) {
				mOnBibleViewListener.onFinishedLoading(BibleView.this);
			}
		}
	}
	
	@Override
	protected void onCreateContextMenu(ContextMenu menu) {
	    super.onCreateContextMenu(menu);
	    
	    if (mFragment == null)
	    	return;
	    
	    if (mSkipNextContextMenu) {
	    	mSkipNextContextMenu = false;
	    	return;
	    }
	    
	    HitTestResult result = getHitTestResult();
	    if (result.getType() == HitTestResult.SRC_ANCHOR_TYPE) {
	    	final int ID_APRI = 0;
	    	final int ID_APRI_NUOVO_PANNELLO = 1;
	    	final int ID_APRI_PANNELLO_ESISTENTE = 2;
	    	final int ID_APRI_POPUP = 3;
	    	
	    	final String url = result.getExtra();
		    MenuItem.OnMenuItemClickListener handler = new MenuItem.OnMenuItemClickListener() {
		        public boolean onMenuItemClick(MenuItem item) {
		        	if (item.getItemId() == ID_APRI) {
						mLaParolaBrowser.vaiAdUrl(url);
					} else if (item.getItemId() == ID_APRI_NUOVO_PANNELLO) {
			        	mParentActivity.openInNewPanel(url, mLaParolaBrowser.getVersione());
					} else if (item.getItemId() == ID_APRI_PANNELLO_ESISTENTE) {
			        	mParentActivity.selectPanelForOpening(url, mLaParolaBrowser.getVersione());
					} else if (item.getItemId() == ID_APRI_POPUP) {
			        	mParentActivity.openInPopupWindow(url, mLaParolaBrowser.getVersione());
			        }
					return true;
		        }
		    };
	    	
	        menu.add(0, ID_APRI, 0, R.string.open_here).setOnMenuItemClickListener(handler);
	        if (mParentActivity.getPanesNumber() < LaParolaActivity.MAX_PANELS) {
	        	menu.add(0, ID_APRI_NUOVO_PANNELLO, 0, R.string.open_new_panel).setOnMenuItemClickListener(handler);
	        }
	        if (mParentActivity.getPanesNumber() > 1) {
	        	menu.add(0, ID_APRI_PANNELLO_ESISTENTE, 0, R.string.open_in_panel).setOnMenuItemClickListener(handler);
	        }
	        menu.add(0, ID_APRI_POPUP, 0, R.string.open_popup).setOnMenuItemClickListener(handler);
	    } else {
	    	final int ID_CHIUDI = 10;
	    	final int ID_APRI_NUOVO_PANNELLO = 11;
	    	final int ID_APRI_PANNELLO_ESISTENTE = 12;
	    	final int ID_GESTIONE_PANNELLI = 13;
	    	final int ID_SELEZIONE = 14;
	    	
		    MenuItem.OnMenuItemClickListener handler = new MenuItem.OnMenuItemClickListener() {
		        public boolean onMenuItemClick(MenuItem item) {
		        	if (item.getItemId() == ID_CHIUDI) {
						mParentActivity.closeActivePanel();
					} else if (item.getItemId() == ID_APRI_NUOVO_PANNELLO) {
			        	mParentActivity.openInNewPanel(getBrowser().getUrlCorrente());
					} else if (item.getItemId() == ID_APRI_PANNELLO_ESISTENTE) {
			        	mParentActivity.switchPanels();
					} else if (item.getItemId() == ID_GESTIONE_PANNELLI) {
			        	mParentActivity.showPanelsManagment();
			        } else if (item.getItemId() == ID_SELEZIONE) {
			        	selectAndCopyText();
			        }
					return true;
		        }
		    };
	    	
	        if (mParentActivity.getPanesNumber() > 1) {
	        	menu.add(0, ID_CHIUDI, 0, R.string.close_panel).setOnMenuItemClickListener(handler);
	        }
	        if (mParentActivity.getPanesNumber() < LaParolaActivity.MAX_PANELS) {
	        	menu.add(0, ID_APRI_NUOVO_PANNELLO, 0, R.string.duplicate_panel).setOnMenuItemClickListener(handler);
	        }
	        if (mParentActivity.getPanesNumber() > 1) {
	        	menu.add(0, ID_APRI_PANNELLO_ESISTENTE, 0, R.string.switch_panel).setOnMenuItemClickListener(handler);
	        }
        	menu.add(0, ID_GESTIONE_PANNELLI, 0, R.string.fragments_management).setOnMenuItemClickListener(handler);
    		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) {
            	menu.add(0, ID_SELEZIONE, 0, R.string.select_text).setOnMenuItemClickListener(handler);
            }
	    }
	}
}

